@@ -45,9 +45,9 @@ class ProxyCube< Cube<eT> >
   typedef const eT*                                ea_type;
   typedef const Cube<eT>&                          aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<eT>& Q;
   
@@ -63,9 +63,9 @@ class ProxyCube< Cube<eT> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
@@ -73,6 +73,9 @@ class ProxyCube< Cube<eT> >
   template<typename eT2>
   arma_inline bool is_alias(const Cube<eT2>& X) const { return (void_ptr(&Q) == void_ptr(&X)); }
   
+  template<typename eT2>
+  arma_inline bool has_overlap(const subview_cube<eT2>& X) const { return is_alias(X.m); }
+  
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
 
@@ -89,9 +92,9 @@ class ProxyCube< GenCube<eT, gen_type> >
   typedef const GenCube<eT, gen_type>&             ea_type;
   typedef const GenCube<eT, gen_type>&             aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const GenCube<eT, gen_type>& Q;
   
@@ -107,15 +110,18 @@ class ProxyCube< GenCube<eT, gen_type> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;                   }
   arma_inline uword get_n_elem()       const { return Q.n_rows*Q.n_cols*Q.n_slices; }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q[i];                  }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q[i];          }
   
   arma_inline         ea_type         get_ea() const { return Q; }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q; }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return GenCube<eT, gen_type>::is_simple; }
   };
@@ -133,9 +139,9 @@ class ProxyCube< GenCube<eT, gen_randu> >
   typedef const eT*                                ea_type;
   typedef const Cube<eT>&                          aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<eT> Q;
   
@@ -151,15 +157,18 @@ class ProxyCube< GenCube<eT, gen_randu> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
@@ -177,9 +186,9 @@ class ProxyCube< GenCube<eT, gen_randn> >
   typedef const eT*                                ea_type;
   typedef const Cube<eT>&                          aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<eT> Q;
   
@@ -195,15 +204,18 @@ class ProxyCube< GenCube<eT, gen_randn> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
@@ -221,9 +233,9 @@ class ProxyCube< OpCube<T1, op_type> >
   typedef const elem_type*                         ea_type;
   typedef const Cube<elem_type>&                   aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<elem_type> Q;
   
@@ -239,15 +251,18 @@ class ProxyCube< OpCube<T1, op_type> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
@@ -265,9 +280,9 @@ class ProxyCube< GlueCube<T1, T2, glue_type> >
   typedef const elem_type*                         ea_type;
   typedef const Cube<elem_type>&                   aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<elem_type> Q;
   
@@ -283,15 +298,18 @@ class ProxyCube< GlueCube<T1, T2, glue_type> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
@@ -309,9 +327,9 @@ class ProxyCube< subview_cube<eT> >
   typedef const subview_cube<eT>&                  ea_type;
   typedef const subview_cube<eT>&                  aligned_ea_type;
   
-  static const bool use_at      = true;
-  static const bool use_mp      = false;
-  static const bool has_subview = true;
+  static constexpr bool use_at      = true;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = true;
   
   arma_aligned const subview_cube<eT>& Q;
   
@@ -327,9 +345,9 @@ class ProxyCube< subview_cube<eT> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q; }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q; }
@@ -337,7 +355,57 @@ class ProxyCube< subview_cube<eT> >
   template<typename eT2>
   arma_inline bool is_alias(const Cube<eT2>& X) const { return (void_ptr(&(Q.m)) == void_ptr(&X)); }
   
-  arma_inline bool is_aligned() const { return false; }
+  template<typename eT2>
+  arma_inline bool has_overlap(const subview_cube<eT2>& X) const { return Q.check_overlap(X); }
+  
+  constexpr bool is_aligned() const { return false; }
+  };
+
+
+
+template<typename eT, typename T1>
+class ProxyCube< subview_cube_slices<eT,T1> >
+  {
+  public:
+  
+  typedef eT                                       elem_type;
+  typedef typename get_pod_type<elem_type>::result pod_type;
+  typedef Cube<eT>                                 stored_type;
+  typedef const eT*                                ea_type;
+  typedef const Cube<eT>&                          aligned_ea_type;
+  
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
+  
+  arma_aligned const Cube<eT> Q;
+  
+  inline explicit ProxyCube(const subview_cube_slices<eT,T1>& A)
+    : Q(A)
+    {
+    arma_extra_debug_sigprint();
+    }
+  
+  arma_inline uword get_n_rows()       const { return Q.n_rows;       }
+  arma_inline uword get_n_cols()       const { return Q.n_cols;       }
+  arma_inline uword get_n_elem_slice() const { return Q.n_elem_slice; }
+  arma_inline uword get_n_slices()     const { return Q.n_slices;     }
+  arma_inline uword get_n_elem()       const { return Q.n_elem;       }
+  
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
+  
+  arma_inline         ea_type         get_ea() const { return Q.memptr(); }
+  arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
+  
+  template<typename eT2>
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
+  
+  arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
 
 
@@ -353,9 +421,9 @@ class ProxyCube< eOpCube<T1, eop_type > >
   typedef const eOpCube<T1, eop_type>&             ea_type;
   typedef const eOpCube<T1, eop_type>&             aligned_ea_type;
   
-  static const bool use_at      = eOpCube<T1, eop_type>::use_at;
-  static const bool use_mp      = eOpCube<T1, eop_type>::use_mp;
-  static const bool has_subview = eOpCube<T1, eop_type>::has_subview;
+  static constexpr bool use_at      = eOpCube<T1, eop_type>::use_at;
+  static constexpr bool use_mp      = eOpCube<T1, eop_type>::use_mp;
+  static constexpr bool has_subview = eOpCube<T1, eop_type>::has_subview;
   
   arma_aligned const eOpCube<T1, eop_type>& Q;
   
@@ -371,9 +439,9 @@ class ProxyCube< eOpCube<T1, eop_type > >
   arma_inline uword get_n_slices()     const { return Q.get_n_slices();     }
   arma_inline uword get_n_elem()       const { return Q.get_n_elem();       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q; }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q; }
@@ -381,6 +449,9 @@ class ProxyCube< eOpCube<T1, eop_type > >
   template<typename eT2>
   arma_inline bool is_alias(const Cube<eT2>& X) const { return Q.P.is_alias(X); }
   
+  template<typename eT2>
+  arma_inline bool has_overlap(const subview_cube<eT2>& X) const { return Q.P.has_overlap(X); }
+  
   arma_inline bool is_aligned() const { return Q.P.is_aligned(); }
   };
 
@@ -397,9 +468,9 @@ class ProxyCube< eGlueCube<T1, T2, eglue_type > >
   typedef const eGlueCube<T1, T2, eglue_type>&     ea_type;
   typedef const eGlueCube<T1, T2, eglue_type>&     aligned_ea_type;
   
-  static const bool use_at      = eGlueCube<T1, T2, eglue_type>::use_at;
-  static const bool use_mp      = eGlueCube<T1, T2, eglue_type>::use_mp;
-  static const bool has_subview = eGlueCube<T1, T2, eglue_type>::has_subview;
+  static constexpr bool use_at      = eGlueCube<T1, T2, eglue_type>::use_at;
+  static constexpr bool use_mp      = eGlueCube<T1, T2, eglue_type>::use_mp;
+  static constexpr bool has_subview = eGlueCube<T1, T2, eglue_type>::has_subview;
   
   arma_aligned const eGlueCube<T1, T2, eglue_type>& Q;
   
@@ -415,9 +486,9 @@ class ProxyCube< eGlueCube<T1, T2, eglue_type > >
   arma_inline uword get_n_slices()     const { return Q.get_n_slices();     }
   arma_inline uword get_n_elem()       const { return Q.get_n_elem();       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q; }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q; }
@@ -425,6 +496,9 @@ class ProxyCube< eGlueCube<T1, T2, eglue_type > >
   template<typename eT2>
   arma_inline bool is_alias(const Cube<eT2>& X) const { return (Q.P1.is_alias(X) || Q.P2.is_alias(X)); }
   
+  template<typename eT2>
+  arma_inline bool has_overlap(const subview_cube<eT2>& X) const { return (Q.P1.has_overlap(X) || Q.P2.has_overlap(X)); }
+  
   arma_inline bool is_aligned() const { return Q.P1.is_aligned() && Q.P2.is_aligned(); }
   };
 
@@ -441,9 +515,9 @@ class ProxyCube< mtOpCube<out_eT, T1, op_type> >
   typedef          const elem_type*             ea_type;
   typedef          const Cube<out_eT>&          aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<out_eT> Q;
   
@@ -459,15 +533,18 @@ class ProxyCube< mtOpCube<out_eT, T1, op_type> >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
@@ -485,9 +562,9 @@ class ProxyCube< mtGlueCube<out_eT, T1, T2, glue_type > >
   typedef          const elem_type*             ea_type;
   typedef          const Cube<out_eT>&          aligned_ea_type;
   
-  static const bool use_at      = false;
-  static const bool use_mp      = false;
-  static const bool has_subview = false;
+  static constexpr bool use_at      = false;
+  static constexpr bool use_mp      = false;
+  static constexpr bool has_subview = false;
   
   arma_aligned const Cube<out_eT> Q;
   
@@ -503,15 +580,18 @@ class ProxyCube< mtGlueCube<out_eT, T1, T2, glue_type > >
   arma_inline uword get_n_slices()     const { return Q.n_slices;     }
   arma_inline uword get_n_elem()       const { return Q.n_elem;       }
   
-  arma_inline elem_type operator[] (const uword i)                                       const { return Q[i];                  }
-  arma_inline elem_type at         (const uword row, const uword col, const uword slice) const { return Q.at(row, col, slice); }
-  arma_inline elem_type at_alt     (const uword i)                                       const { return Q.at_alt(i);           }
+  arma_inline elem_type operator[] (const uword i)                               const { return Q[i];          }
+  arma_inline elem_type at         (const uword r, const uword c, const uword s) const { return Q.at(r, c, s); }
+  arma_inline elem_type at_alt     (const uword i)                               const { return Q.at_alt(i);   }
   
   arma_inline         ea_type         get_ea() const { return Q.memptr(); }
   arma_inline aligned_ea_type get_aligned_ea() const { return Q;          }
   
   template<typename eT2>
-  arma_inline bool is_alias(const Cube<eT2>&) const { return false; }
+  constexpr bool is_alias(const Cube<eT2>&) const { return false; }
+  
+  template<typename eT2>
+  constexpr bool has_overlap(const subview_cube<eT2>&) const { return false; }
   
   arma_inline bool is_aligned() const { return memory::is_aligned(Q.memptr()); }
   };
