@@ -54,17 +54,17 @@
 //! @{
 
 
-template<typename cx_type, uword fixed_N, bool> struct store {};
+template<typename cx_type, uword fixed_N, bool> struct fft_store {};
 
 template<typename cx_type, uword fixed_N>
-struct store<cx_type, fixed_N, true>
+struct fft_store<cx_type, fixed_N, true>
   {
-  static const uword N = fixed_N;
+  static constexpr uword N = fixed_N;
   
   arma_aligned cx_type coeffs_array[fixed_N];
   
-  inline store()      {}
-  inline store(uword) {}
+  inline fft_store()      {}
+  inline fft_store(uword) {}
   
   arma_inline       cx_type* coeffs_ptr()       { return &coeffs_array[0]; }
   arma_inline const cx_type* coeffs_ptr() const { return &coeffs_array[0]; }
@@ -73,14 +73,14 @@ struct store<cx_type, fixed_N, true>
 
 
 template<typename cx_type, uword fixed_N>
-struct store<cx_type, fixed_N, false>
+struct fft_store<cx_type, fixed_N, false>
   {
   const uword N;
   
   podarray<cx_type> coeffs_array;
   
-  inline store()           : N(0)    {}
-  inline store(uword in_N) : N(in_N) { coeffs_array.set_size(N); }
+  inline fft_store()           : N(0)    {}
+  inline fft_store(uword in_N) : N(in_N) { coeffs_array.set_size(N); }
   
   arma_inline       cx_type* coeffs_ptr()       { return coeffs_array.memptr(); }
   arma_inline const cx_type* coeffs_ptr() const { return coeffs_array.memptr(); }
@@ -89,14 +89,14 @@ struct store<cx_type, fixed_N, false>
 
 
 template<typename cx_type, bool inverse, uword fixed_N = 0>
-class fft_engine : public store<cx_type, fixed_N, (fixed_N > 0)>
+class fft_engine : public fft_store<cx_type, fixed_N, (fixed_N > 0)>
   {
   public:
   
   typedef typename get_pod_type<cx_type>::result T;
   
-  using store<cx_type, fixed_N, (fixed_N > 0)>::N;
-  using store<cx_type, fixed_N, (fixed_N > 0)>::coeffs_ptr;
+  using fft_store<cx_type, fixed_N, (fixed_N > 0)>::N;
+  using fft_store<cx_type, fixed_N, (fixed_N > 0)>::coeffs_ptr;
   
   podarray<uword>   residue;
   podarray<uword>   radix;
@@ -141,7 +141,7 @@ class fft_engine : public store<cx_type, fixed_N, (fixed_N > 0)>
   
   inline
   fft_engine(const uword in_N)
-    : store< cx_type, fixed_N, (fixed_N > 0) >(in_N)
+    : fft_store< cx_type, fixed_N, (fixed_N > 0) >(in_N)
     {
     arma_extra_debug_sigprint();
     
