@@ -21,16 +21,14 @@
 //! Eigenvalues of real/complex symmetric/hermitian matrix X
 template<typename T1>
 inline
-bool
+typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, bool >::result
 eig_sym
   (
          Col<typename T1::pod_type>&     eigval,
-  const Base<typename T1::elem_type,T1>& X,
-  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  const Base<typename T1::elem_type,T1>& X
   )
   {
   arma_extra_debug_sigprint();
-  arma_ignore(junk);
   
   // unwrap_check not used as T1::elem_type and T1::pod_type may not be the same.
   // furthermore, it doesn't matter if X is an alias of eigval, as auxlib::eig_sym() makes a copy of X
@@ -52,15 +50,13 @@ eig_sym
 template<typename T1>
 arma_warn_unused
 inline
-Col<typename T1::pod_type>
+typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, Col<typename T1::pod_type> >::result
 eig_sym
   (
-  const Base<typename T1::elem_type,T1>& X,
-  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  const Base<typename T1::elem_type,T1>& X
   )
   {
   arma_extra_debug_sigprint();
-  arma_ignore(junk);
   
   Col<typename T1::pod_type> out;
   const bool status = auxlib::eig_sym(out, X);
@@ -76,41 +72,74 @@ eig_sym
 
 
 
+//! internal helper function
+template<typename eT>
+inline
+bool
+eig_sym_helper
+  (
+        Col<typename get_pod_type<eT>::result>& eigval,
+        Mat<eT>&                                eigvec,
+  const Mat<eT>&                                X,
+  const char                                    method_sig,
+  const char*                                   caller_sig
+  )
+  {
+  arma_extra_debug_sigprint();
+  
+  // if(auxlib::rudimentary_sym_check(X) == false)
+  //   {
+  //   if(is_cx<eT>::no )  { arma_debug_warn(caller_sig, ": given matrix is not symmetric"); }
+  //   if(is_cx<eT>::yes)  { arma_debug_warn(caller_sig, ": given matrix is not hermitian"); }
+  //   return false;
+  //   }
+  
+  if((arma_config::debug) && (auxlib::rudimentary_sym_check(X) == false))
+    {
+    if(is_cx<eT>::no )  { arma_debug_warn(caller_sig, ": given matrix is not symmetric"); }
+    if(is_cx<eT>::yes)  { arma_debug_warn(caller_sig, ": given matrix is not hermitian"); }
+    }
+  
+  bool status = false;
+  
+  if(method_sig == 'd') { status = auxlib::eig_sym_dc(eigval, eigvec, X); }
+  
+  if(status == false)   { status = auxlib::eig_sym(eigval, eigvec, X);    }
+  
+  return status;
+  }
+
+
+
 //! Eigenvalues and eigenvectors of real/complex symmetric/hermitian matrix X
 template<typename T1> 
 inline
-bool
+typename enable_if2< is_supported_blas_type<typename T1::elem_type>::value, bool >::result
 eig_sym
   (
          Col<typename T1::pod_type>&     eigval,
          Mat<typename T1::elem_type>&    eigvec,
-  const Base<typename T1::elem_type,T1>& X,
-  const char* method =                   "dc",
-  const typename arma_blas_type_only<typename T1::elem_type>::result* junk = 0
+  const Base<typename T1::elem_type,T1>& expr,
+  const char* method =                   "dc"
   )
   {
   arma_extra_debug_sigprint();
-  arma_ignore(junk);
   
   typedef typename T1::elem_type eT;
   
-  const char sig = (method != NULL) ? method[0] : char(0);
+  const char sig = (method != nullptr) ? method[0] : char(0);
   
   arma_debug_check( ((sig != 's') && (sig != 'd')),         "eig_sym(): unknown method specified"                             );
   arma_debug_check( void_ptr(&eigval) == void_ptr(&eigvec), "eig_sym(): parameter 'eigval' is an alias of parameter 'eigvec'" );
   
-  const Proxy<T1> P(X.get_ref());
+  const quasi_unwrap<T1> U(expr.get_ref());
   
-  const bool is_alias = P.is_alias(eigvec);
+  const bool is_alias = U.is_alias(eigvec);
   
   Mat<eT>  eigvec_tmp;
   Mat<eT>& eigvec_out = (is_alias == false) ? eigvec : eigvec_tmp;
   
-  bool status = false;
-  
-  if(sig == 'd')       { status = auxlib::eig_sym_dc(eigval, eigvec_out, P.Q); }
-  
-  if(status == false)  { status = auxlib::eig_sym(eigval, eigvec_out, P.Q);    }
+  const bool status = eig_sym_helper(eigval, eigvec_out, U.M, sig, "eig_sym()");
   
   if(status == false)
     {
