@@ -1,11 +1,17 @@
-// Copyright (C) 2013 National ICT Australia (NICTA)
+// Copyright 2008-2016 Conrad Sanderson (http://conradsanderson.id.au)
+// Copyright 2008-2016 National ICT Australia (NICTA)
 // 
-// This Source Code Form is subject to the terms of the Mozilla Public
-// License, v. 2.0. If a copy of the MPL was not distributed with this
-// file, You can obtain one at http://mozilla.org/MPL/2.0/.
-// -------------------------------------------------------------------
+// Licensed under the Apache License, Version 2.0 (the "License");
+// you may not use this file except in compliance with the License.
+// You may obtain a copy of the License at
+// http://www.apache.org/licenses/LICENSE-2.0
 // 
-// Written by Conrad Sanderson - http://conradsanderson.id.au
+// Unless required by applicable law or agreed to in writing, software
+// distributed under the License is distributed on an "AS IS" BASIS,
+// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+// See the License for the specific language governing permissions and
+// limitations under the License.
+// ------------------------------------------------------------------------
 
 
 //! \addtogroup fn_fft2
@@ -18,6 +24,7 @@
 
 
 template<typename T1>
+arma_warn_unused
 inline
 typename
 enable_if2
@@ -46,6 +53,7 @@ fft2(const T1& A)
 
 
 template<typename T1>
+arma_warn_unused
 inline
 typename
 enable_if2
@@ -59,22 +67,23 @@ fft2(const T1& A, const uword n_rows, const uword n_cols)
   
   typedef typename T1::elem_type eT;
   
-  const unwrap<T1>   tmp(A);
-  const Mat<eT>& B = tmp.M;
+  const quasi_unwrap<T1> tmp(A);
+  const Mat<eT>&     B = tmp.M;
   
   const bool do_resize = (B.n_rows != n_rows) || (B.n_cols != n_cols);
   
-  return fft2( do_resize ? resize(B,n_rows,n_cols) : B );
+  return (do_resize) ? fft2(resize(B,n_rows,n_cols)) : fft2(B);
   }
 
 
 
 template<typename T1>
+arma_warn_unused
 inline
 typename
 enable_if2
   <
-  (is_arma_type<T1>::value && is_complex_strict<typename T1::elem_type>::value),
+  (is_arma_type<T1>::value && (is_cx_float<typename T1::elem_type>::yes || is_cx_double<typename T1::elem_type>::yes)),
   Mat< std::complex<typename T1::pod_type> >
   >::result
 ifft2(const T1& A)
@@ -98,11 +107,12 @@ ifft2(const T1& A)
 
 
 template<typename T1>
+arma_warn_unused
 inline
 typename
 enable_if2
   <
-  (is_arma_type<T1>::value && is_complex_strict<typename T1::elem_type>::value),
+  (is_arma_type<T1>::value && (is_cx_float<typename T1::elem_type>::yes || is_cx_double<typename T1::elem_type>::yes)),
   Mat< std::complex<typename T1::pod_type> >
   >::result
 ifft2(const T1& A, const uword n_rows, const uword n_cols)
@@ -111,12 +121,12 @@ ifft2(const T1& A, const uword n_rows, const uword n_cols)
   
   typedef typename T1::elem_type eT;
   
-  const unwrap<T1>   tmp(A);
-  const Mat<eT>& B = tmp.M;
+  const quasi_unwrap<T1> tmp(A);
+  const Mat<eT>&     B = tmp.M;
   
   const bool do_resize = (B.n_rows != n_rows) || (B.n_cols != n_cols);
   
-  return ifft2( do_resize ? resize(B,n_rows,n_cols) : B );
+  return (do_resize) ? ifft2(resize(B,n_rows,n_cols)) : ifft2(B);
   }
 
 
