@@ -26,8 +26,8 @@ const Op<T1, op_strans>
 strans
   (
   const T1& X,
-  const typename enable_if< is_arma_type<T1>::value == true >::result* junk1 = 0,
-  const typename arma_cx_only<typename T1::elem_type>::result*         junk2 = 0
+  const typename enable_if< is_arma_type<T1>::value >::result* junk1 = nullptr,
+  const typename arma_cx_only<typename T1::elem_type>::result* junk2 = nullptr
   )
   {
   arma_extra_debug_sigprint();
@@ -48,8 +48,8 @@ const Op<T1, op_htrans>
 strans
   (
   const T1& X,
-  const typename enable_if< is_arma_type<T1>::value == true >::result* junk1 = 0,
-  const typename arma_not_cx<typename T1::elem_type>::result*          junk2 = 0
+  const typename enable_if< is_arma_type<T1>::value >::result* junk1 = nullptr,
+  const typename arma_not_cx<typename T1::elem_type>::result*  junk2 = nullptr
   )
   {
   arma_extra_debug_sigprint();
@@ -67,18 +67,40 @@ strans
 
 template<typename T1>
 arma_warn_unused
-inline
-typename
-enable_if2
-  <
-  is_arma_sparse_type<T1>::value,
-  const SpOp<T1,spop_strans>
-  >::result
-strans(const T1& x)
+arma_inline
+const SpOp<T1, spop_strans>
+strans
+  (
+  const T1& X,
+  const typename enable_if< is_arma_sparse_type<T1>::value >::result* junk1 = nullptr,
+  const typename arma_cx_only<typename T1::elem_type>::result*        junk2 = nullptr
+  )
   {
   arma_extra_debug_sigprint();
+  arma_ignore(junk1);
+  arma_ignore(junk2);
   
-  return SpOp<T1,spop_strans>(x);
+  return SpOp<T1, spop_strans>(X);
+  }
+
+
+
+template<typename T1>
+arma_warn_unused
+arma_inline
+const SpOp<T1, spop_htrans>
+strans
+  (
+  const T1& X,
+  const typename enable_if< is_arma_sparse_type<T1>::value >::result* junk1 = nullptr,
+  const typename arma_not_cx<typename T1::elem_type>::result*         junk2 = nullptr
+  )
+  {
+  arma_extra_debug_sigprint();
+  arma_ignore(junk1);
+  arma_ignore(junk2);
+  
+  return SpOp<T1, spop_htrans>(X);
   }
 
 
