@@ -158,7 +158,7 @@ gmm_diag<eT>::set_params(const Base<eT,T1>& in_means_expr, const Base<eT,T2>& in
   
   const eT s = accu(in_hefts);
   
-  arma_debug_check( ((s < (eT(1) - Datum<eT>::eps)) || (s > (eT(1) + Datum<eT>::eps))), "gmm_diag::set_params(): sum of given hefts is not 1" );
+  arma_debug_check( ((s < (eT(1) - eT(0.001))) || (s > (eT(1) + eT(0.001)))), "gmm_diag::set_params(): sum of given hefts is not 1" );
   
   access::rw(means) = in_means;
   access::rw(dcovs) = in_dcovs;
@@ -181,8 +181,8 @@ gmm_diag<eT>::set_means(const Base<eT,T1>& in_means_expr)
   
   const Mat<eT>& in_means = tmp.M;
   
-  arma_debug_check( (size(in_means) != size(means)), "gmm_diag::set_means(): given means have incompatible size" );
-  arma_debug_check( (in_means.is_finite() == false), "gmm_diag::set_means(): given means have non-finite values" );
+  arma_debug_check( (arma::size(in_means) != arma::size(means)), "gmm_diag::set_means(): given means have incompatible size" );
+  arma_debug_check( (in_means.is_finite() == false),             "gmm_diag::set_means(): given means have non-finite values" );
   
   access::rw(means) = in_means;
   }
@@ -201,9 +201,9 @@ gmm_diag<eT>::set_dcovs(const Base<eT,T1>& in_dcovs_expr)
   
   const Mat<eT>& in_dcovs = tmp.M;
   
-  arma_debug_check( (size(in_dcovs) != size(dcovs)),     "gmm_diag::set_dcovs(): given dcovs have incompatible size"       );
-  arma_debug_check( (in_dcovs.is_finite() == false),     "gmm_diag::set_dcovs(): given dcovs have non-finite values"       );
-  arma_debug_check( (any(vectorise(in_dcovs) <= eT(0))), "gmm_diag::set_dcovs(): given dcovs have negative or zero values" );
+  arma_debug_check( (arma::size(in_dcovs) != arma::size(dcovs)), "gmm_diag::set_dcovs(): given dcovs have incompatible size"       );
+  arma_debug_check( (in_dcovs.is_finite() == false),             "gmm_diag::set_dcovs(): given dcovs have non-finite values"       );
+  arma_debug_check( (any(vectorise(in_dcovs) <= eT(0))),         "gmm_diag::set_dcovs(): given dcovs have negative or zero values" );
   
   access::rw(dcovs) = in_dcovs;
   
@@ -224,13 +224,13 @@ gmm_diag<eT>::set_hefts(const Base<eT,T1>& in_hefts_expr)
   
   const Mat<eT>& in_hefts = tmp.M;
   
-  arma_debug_check( (size(in_hefts) != size(hefts)),     "gmm_diag::set_hefts(): given hefts have incompatible size" );
-  arma_debug_check( (in_hefts.is_finite() == false),     "gmm_diag::set_hefts(): given hefts have non-finite values" );
-  arma_debug_check( (any(vectorise(in_hefts) <  eT(0))), "gmm_diag::set_hefts(): given hefts have negative values"   );
+  arma_debug_check( (arma::size(in_hefts) != arma::size(hefts)), "gmm_diag::set_hefts(): given hefts have incompatible size" );
+  arma_debug_check( (in_hefts.is_finite() == false),             "gmm_diag::set_hefts(): given hefts have non-finite values" );
+  arma_debug_check( (any(vectorise(in_hefts) <  eT(0))),         "gmm_diag::set_hefts(): given hefts have negative values"   );
   
   const eT s = accu(in_hefts);
   
-  arma_debug_check( ((s < (eT(1) - Datum<eT>::eps)) || (s > (eT(1) + Datum<eT>::eps))), "gmm_diag::set_hefts(): sum of given hefts is not 1" );
+  arma_debug_check( ((s < (eT(1) - eT(0.001))) || (s > (eT(1) + eT(0.001)))), "gmm_diag::set_hefts(): sum of given hefts is not 1" );
   
   // make sure all hefts are positive and non-zero
   
@@ -457,19 +457,11 @@ gmm_diag<eT>::log_p(const T1& expr, const gmm_empty_arg& junk1, typename enable_
   arma_ignore(junk1);
   arma_ignore(junk2);
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >(expr);
-    
-    return internal_vec_log_p(X);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr);
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_vec_log_p(X);
-    }
+  const quasi_unwrap<T1> tmp(expr);
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_vec_log_p(X);
   }
 
 
@@ -483,19 +475,11 @@ gmm_diag<eT>::log_p(const T1& expr, const uword gaus_id, typename enable_if<((is
   arma_extra_debug_sigprint();
   arma_ignore(junk2);
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >(expr);
-    
-    return internal_vec_log_p(X, gaus_id);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr);
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_vec_log_p(X, gaus_id);
-    }
+  const quasi_unwrap<T1> tmp(expr);
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_vec_log_p(X, gaus_id);
   }
 
 
@@ -508,19 +492,11 @@ gmm_diag<eT>::sum_log_p(const Base<eT,T1>& expr) const
   {
   arma_extra_debug_sigprint();
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >( expr.get_ref() );
-    
-    return internal_sum_log_p(X);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr.get_ref());
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_sum_log_p(X);
-    }
+  const quasi_unwrap<T1> tmp(expr.get_ref());
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_sum_log_p(X);
   }
 
 
@@ -533,19 +509,11 @@ gmm_diag<eT>::sum_log_p(const Base<eT,T1>& expr, const uword gaus_id) const
   {
   arma_extra_debug_sigprint();
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >( expr.get_ref() );
-    
-    return internal_sum_log_p(X, gaus_id);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr.get_ref());
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_sum_log_p(X, gaus_id);
-    }
+  const quasi_unwrap<T1> tmp(expr.get_ref());
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_sum_log_p(X, gaus_id);
   }
 
 
@@ -558,19 +526,11 @@ gmm_diag<eT>::avg_log_p(const Base<eT,T1>& expr) const
   {
   arma_extra_debug_sigprint();
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >( expr.get_ref() );
-    
-    return internal_avg_log_p(X);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr.get_ref());
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_avg_log_p(X);
-    }
+  const quasi_unwrap<T1> tmp(expr.get_ref());
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_avg_log_p(X);
   }
 
 
@@ -583,19 +543,11 @@ gmm_diag<eT>::avg_log_p(const Base<eT,T1>& expr, const uword gaus_id) const
   {
   arma_extra_debug_sigprint();
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >( expr.get_ref() );
-    
-    return internal_avg_log_p(X, gaus_id);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr.get_ref());
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_avg_log_p(X, gaus_id);
-    }
+  const quasi_unwrap<T1> tmp(expr.get_ref());
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_avg_log_p(X, gaus_id);
   }
 
 
@@ -609,19 +561,11 @@ gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist, typename enable_
   arma_extra_debug_sigprint();
   arma_ignore(junk);
   
-  if(is_subview_col<T1>::value)
-    {
-    const subview_col<eT>& X = reinterpret_cast< const subview_col<eT>& >(expr);
-    
-    return internal_scalar_assign(X, dist);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr);
-    const Mat<eT>& X = tmp.M;
-    
-    return internal_scalar_assign(X, dist);
-    }
+  const quasi_unwrap<T1> tmp(expr);
+  
+  const Mat<eT>& X = tmp.M;
+  
+  return internal_scalar_assign(X, dist);
   }
 
 
@@ -637,19 +581,11 @@ gmm_diag<eT>::assign(const T1& expr, const gmm_dist_mode& dist, typename enable_
   
   urowvec out;
   
-  if(is_subview<T1>::value)
-    {
-    const subview<eT>& X = reinterpret_cast< const subview<eT>& >(expr);
-    
-    internal_vec_assign(out, X, dist);
-    }
-  else
-    {
-    const unwrap<T1>   tmp(expr);
-    const Mat<eT>& X = tmp.M;
-    
-    internal_vec_assign(out, X, dist);
-    }
+  const quasi_unwrap<T1> tmp(expr);
+  
+  const Mat<eT>& X = tmp.M;
+  
+  internal_vec_assign(out, X, dist);
   
   return out;
   }
@@ -1077,7 +1013,7 @@ gmm_diag<eT>::internal_gen_boundaries(const uword N) const
     const uword n_threads_avail = (omp_in_parallel()) ? uword(1) : uword(omp_get_max_threads());
     const uword n_threads       = (n_threads_avail > 0) ? ( (n_threads_avail <= N) ? n_threads_avail : 1 ) : 1;
   #else
-    static const uword n_threads = 1;
+    static constexpr uword n_threads = 1;
   #endif
   
   // get_cout_stream() << "gmm_diag::internal_gen_boundaries(): n_threads: " << n_threads << '\n';
@@ -1189,10 +1125,9 @@ gmm_diag<eT>::internal_scalar_log_p(const eT* x, const uword g) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 Row<eT>
-gmm_diag<eT>::internal_vec_log_p(const T1& X) const
+gmm_diag<eT>::internal_vec_log_p(const Mat<eT>& X) const
   {
   arma_extra_debug_sigprint();
   
@@ -1242,10 +1177,9 @@ gmm_diag<eT>::internal_vec_log_p(const T1& X) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 Row<eT>
-gmm_diag<eT>::internal_vec_log_p(const T1& X, const uword gaus_id) const
+gmm_diag<eT>::internal_vec_log_p(const Mat<eT>& X, const uword gaus_id) const
   {
   arma_extra_debug_sigprint();
   
@@ -1296,10 +1230,9 @@ gmm_diag<eT>::internal_vec_log_p(const T1& X, const uword gaus_id) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 eT
-gmm_diag<eT>::internal_sum_log_p(const T1& X) const
+gmm_diag<eT>::internal_sum_log_p(const Mat<eT>& X) const
   {
   arma_extra_debug_sigprint();
   
@@ -1353,10 +1286,9 @@ gmm_diag<eT>::internal_sum_log_p(const T1& X) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 eT
-gmm_diag<eT>::internal_sum_log_p(const T1& X, const uword gaus_id) const
+gmm_diag<eT>::internal_sum_log_p(const Mat<eT>& X, const uword gaus_id) const
   {
   arma_extra_debug_sigprint();
   
@@ -1411,10 +1343,9 @@ gmm_diag<eT>::internal_sum_log_p(const T1& X, const uword gaus_id) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 eT
-gmm_diag<eT>::internal_avg_log_p(const T1& X) const
+gmm_diag<eT>::internal_avg_log_p(const Mat<eT>& X) const
   {
   arma_extra_debug_sigprint();
   
@@ -1479,10 +1410,9 @@ gmm_diag<eT>::internal_avg_log_p(const T1& X) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 eT
-gmm_diag<eT>::internal_avg_log_p(const T1& X, const uword gaus_id) const
+gmm_diag<eT>::internal_avg_log_p(const Mat<eT>& X, const uword gaus_id) const
   {
   arma_extra_debug_sigprint();
   
@@ -1548,10 +1478,9 @@ gmm_diag<eT>::internal_avg_log_p(const T1& X, const uword gaus_id) const
 
 
 template<typename eT>
-template<typename T1>
 inline
 uword
-gmm_diag<eT>::internal_scalar_assign(const T1& X, const gmm_dist_mode& dist_mode) const
+gmm_diag<eT>::internal_scalar_assign(const Mat<eT>& X, const gmm_dist_mode& dist_mode) const
   {
   arma_extra_debug_sigprint();
   
@@ -1605,10 +1534,9 @@ gmm_diag<eT>::internal_scalar_assign(const T1& X, const gmm_dist_mode& dist_mode
 
 
 template<typename eT>
-template<typename T1>
 inline
 void
-gmm_diag<eT>::internal_vec_assign(urowvec& out, const T1& X, const gmm_dist_mode& dist_mode) const
+gmm_diag<eT>::internal_vec_assign(urowvec& out, const Mat<eT>& X, const gmm_dist_mode& dist_mode) const
   {
   arma_extra_debug_sigprint();
   
@@ -1884,10 +1812,8 @@ gmm_diag<eT>::generate_initial_means(const Mat<eT>& X, const gmm_seed_mode& seed
     {
     uvec initial_indices;
     
-         if(seed_mode == static_subset)  { initial_indices = linspace<uvec>(0, X.n_cols-1, N_gaus);                   }
-    else if(seed_mode == random_subset)  { initial_indices = uvec(sort_index(randu<vec>(X.n_cols))).rows(0,N_gaus-1); }
-    
-    // not using randi() here as on some primitive systems it produces vectors with non-unique values
+         if(seed_mode == static_subset)  { initial_indices = linspace<uvec>(0, X.n_cols-1, N_gaus); }
+    else if(seed_mode == random_subset)  { initial_indices = randperm<uvec>(X.n_cols, N_gaus);      }
     
     // initial_indices.print("initial_indices:");
     
